@@ -12,15 +12,19 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from typing import Optional
+
 import jax
 import jax.numpy as jnp
 
+from inspect import signature
+
 from netket.operator import Squared, AbstractSuperOperator
 from netket.vqs import MCMixedState
-from netket.utils import warn_deprecation
 from netket.optimizer import (
     identity_preconditioner,
     PreconditionerT,
+    _DeprecatedPreconditionerSignature,
 )
 
 from .vmc_common import info
@@ -38,9 +42,7 @@ class SteadyState(AbstractVariationalDriver):
         optimizer,
         *args,
         variational_state: MCMixedState = None,
-        preconditioner: PreconditionerT = None,
-        sr: PreconditionerT = None,
-        sr_restart: bool = None,
+        preconditioner: PreconditionerT = identity_preconditioner,
         **kwargs,
     ):
         """
@@ -62,39 +64,6 @@ class SteadyState(AbstractVariationalDriver):
         if not isinstance(lindbladian, AbstractSuperOperator):
             raise TypeError("The first argument must be a super-operator")
 
-        if sr is not None:
-            if preconditioner is not None:
-                raise ValueError(
-                    "sr is deprecated in favour of preconditioner kwarg. You should not pass both"
-                )
-            else:
-                preconditioner = sr
-                warn_deprecation(
-                    (
-                        "The `sr` keyword argument is deprecated in favour of `preconditioner`."
-                        "Please update your code to `SteadyState(.., preconditioner=your_sr)`"
-                    )
-                )
-
-        if sr_restart is not None:
-            if preconditioner is None:
-                raise ValueError(
-                    "sr_restart only makes sense if you have a preconditioner/SR."
-                )
-            else:
-                preconditioner.solver_restart = sr_restart
-                warn_deprecation(
-                    (
-                        "The `sr_restart` keyword argument is deprecated in favour of specifying "
-                        "`solver_restart` in the constructor of the SR object."
-                        "Please update your code to `SteadyState(.., preconditioner=nk.optimizer.SR(..., solver_restart=True/False))`"
-                    )
-                )
-
-        # move as kwarg once deprecations are removed
-        if preconditioner is None:
-            preconditioner = identity_preconditioner
-
         super().__init__(variational_state, optimizer, minimized_quantity_name="LdagL")
 
         self._lind = lindbladian
@@ -121,7 +90,7 @@ class SteadyState(AbstractVariationalDriver):
 
         # if it's the identity it does
         # self._dp = self._loss_grad
-        self._dp = self.preconditioner(self.state, self._loss_grad)
+        self._dp = self.preconditioner(self.state, self._loss_grad, self.step_count)
 
         # If parameters are real, then take only real part of the gradient (if it's complex)
         self._dp = jax.tree_map(
@@ -132,6 +101,39 @@ class SteadyState(AbstractVariationalDriver):
 
         return self._dp
 
+    @property
+    def preconditioner(self):
+        """
+        The preconditioner used to modify the gradient.
+
+        This is a function with the following signature
+
+        .. code-block:: python
+
+            precondtioner(vstate: VariationalState,
+                          grad: PyTree,
+                          step: Optional[Scalar] = None)
+
+        Where the first argument is a variational state, the second argument
+        is the PyTree of the gradient to precondition and the last optional
+        argument is the step, used to change some parameters along the
+        optimisation.
+
+        Often, this is taken to be :func:`nk.optimizer.SR`. If it is set to
+        `None`, then the identity is used.
+        """
+        return self._preconditioner
+
+    @preconditioner.setter
+    def preconditioner(self, val: Optional[PreconditionerT]):
+        if val is None:
+            val = identity_preconditioner
+
+        if len(signature(val).parameters) == 2:
+            val = _DeprecatedPreconditionerSignature(val)
+
+        self._preconditioner = val
+
     @property
     def ldagl(self):
         """
@@ -140,9 +142,6 @@ class SteadyState(AbstractVariationalDriver):
         """
         return self._loss_stats
 
-    #    def reset(self):
-    #        super().reset()
-
     def __repr__(self):
         return (
             "SteadyState("
