@@ -30,8 +30,10 @@ from . import dynamics
 from . import sampler
 from . import vqs
 from . import logging
+from . import qsr
 
 from .driver import TDVP
+from .qsr import QSR
 
 from netket.utils import _hide_submodules
 
