@@ -56,7 +56,6 @@ def random_state(hilb: Spin, key, batches: int, *, dtype=np.float32):
                 axis=1,
             )
 
-            # deprecated: return jax.random.shuffle(key, x, axis=1)
             return jax.vmap(jax.random.permutation)(
                 jax.random.split(key, x.shape[0]), x
             )
@@ -114,6 +113,7 @@ def _flipat_generic(key, x, i, s):
     r = jax.random.uniform(key)
     xi_new = jax.numpy.floor(r * (n_states - 1)) * 2 - (n_states - 1)
     xi_new = xi_new + 2 * (xi_new >= xi_old)
+    xi_new = xi_new.astype(x.dtype)
 
     new_state = x.at[i].set(xi_new)
     return new_state, xi_old
