@@ -15,7 +15,7 @@
 from .runtime_log import RuntimeLog
 from .json_log import JsonLog
 from .state_log import StateLog
-from .tensorboard import TensorBoardLog, TBLog
+from .tensorboard import TensorBoardLog
 
 from netket.utils import tensorboard_available as _tensorboard_available
 
