@@ -12,14 +12,15 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from typing import Any, Tuple, Callable, Union
+from typing import Tuple, Callable, Union, Optional
 
-import numpy as np
+import jax
+import jax.numpy as jnp
 
 from flax import linen as nn
 from jax.nn.initializers import lecun_normal, zeros
 
-from netket.utils.types import NNInitFunc
+from netket.utils.types import NNInitFunc, DType
 from netket import nn as nknn
 
 default_kernel_init = lecun_normal()
@@ -42,25 +43,25 @@ class MLP(nn.Module):
     """
     output_dim: int = 1
     """The output dimension"""
-    hidden_dims: Tuple[int] = None
+    hidden_dims: Optional[Union[int, Tuple[int, ...]]] = None
     """The size of the hidden layers, excluding the output layer."""
-    hidden_dims_alpha: Tuple[int] = None
+    hidden_dims_alpha: Optional[Union[int, Tuple[int, ...]]] = None
     """The size of the hidden layers provided as number of times the input size.
     One must choose to either specify this or the hidden_dims keyword argument"""
-    param_dtype: Any = np.float64
+    param_dtype: DType = jnp.float64
     """The dtype of the weights."""
-    hidden_activations: Union[Callable, Tuple[Callable]] = nknn.gelu
+    hidden_activations: Optional[Union[Callable, Tuple[Callable, ...]]] = nknn.gelu
     """The nonlinear activation function after each hidden layer.
     Can be provided as a single activation,
     where the same activation will be used for every layer."""
-    output_activation: Callable = None
+    output_activation: Optional[Callable] = None
     """The nonlinear activation at the output layer.
     If None is provided, the output layer will be essentially linear."""
     use_hidden_bias: bool = True
     """If True uses a bias in the hidden layer."""
     use_output_bias: bool = True
     """If True adds a bias to the output layer."""
-    precision: Any = None
+    precision: Optional[jax.lax.Precision] = None
     """Numerical precision of the computation see :class:`jax.lax.Precision` for details."""
     kernel_init: NNInitFunc = default_kernel_init
     """Initializer for the Dense layer matrix."""
