@@ -32,11 +32,6 @@ from netket.graph import Graph, Lattice
 default_equivariant_initializer = lecun_normal(in_axis=1, out_axis=0)
 
 
-def _normalise_mask(mask, new_norm):
-    mask = jnp.asarray(mask)
-    return mask / jnp.linalg.norm(mask) * new_norm**0.5
-
-
 def symm_input_warning(x_shape, new_x_shape, name):
     warn_deprecation(
         (
@@ -62,7 +57,9 @@ class DenseSymmMatrix(Module):
     use_bias: bool = True
     """Whether to add a bias to the output (default: True)."""
     mask: Optional[HashableArray] = None
-    """Mask that zeros out elements of the filter. Should be of shape [inp.shape[-1]]"""
+    """Optional array of shape `(n_sites,)` used to restrict the convolutional
+        kernel. Only parameters with mask :math:'\ne 0' are used. For best performance a
+        boolean mask should be used"""
     param_dtype: Any = jnp.float64
     """The dtype of the weights."""
     precision: Any = None
@@ -77,7 +74,7 @@ class DenseSymmMatrix(Module):
         # pylint: disable=attribute-defined-outside-init
         self.n_symm, self.n_sites = np.asarray(self.symmetries).shape
         if self.mask is not None:
-            self.scaled_mask = _normalise_mask(self.mask, self.n_sites)
+            (self.kernel_indices,) = np.nonzero(self.mask.wrapped)
 
     @compact
     def __call__(self, x: Array) -> Array:
@@ -110,23 +107,25 @@ class DenseSymmMatrix(Module):
             bias = None
 
         if self.mask is not None:
-            scaled_mask = self.scaled_mask
+            kernel_params = self.param(
+                "kernel",
+                self.kernel_init,
+                (self.features, in_features, len(self.kernel_indices)),
+                self.param_dtype,
+            )
+
+            kernel = jnp.zeros(
+                [self.features, in_features, self.n_sites], self.param_dtype
+            )
+            kernel = kernel.at[:, :, self.kernel_indices].set(kernel_params)
         else:
-            scaled_mask = None
-
-        kernel = self.param(
-            "kernel",
-            self.kernel_init,
-            (self.features, in_features, self.n_sites),
-            self.param_dtype,
-        )
-
-        x, scaled_mask, kernel, bias = promote_dtype(
-            x, scaled_mask, kernel, bias, dtype=None
-        )
-
-        if self.mask is not None:
-            kernel = kernel * jnp.expand_dims(scaled_mask, (0, 1))
+            kernel = self.param(
+                "kernel",
+                self.kernel_init,
+                (self.features, in_features, self.n_sites),
+                self.param_dtype,
+            )
+        x, kernel, bias = promote_dtype(x, kernel, bias, dtype=None)
 
         # Converts the convolutional kernel of shape (self.features, in_features, n_sites)
         # to a full dense kernel of shape (self.features, in_features, n_symm, n_sites).
@@ -162,7 +161,9 @@ class DenseSymmFFT(Module):
     use_bias: bool = True
     """Whether to add a bias to the output (default: True)."""
     mask: Optional[HashableArray] = None
-    """Mask that zeros out elements of the filter. Should be of shape [inp.shape[-1]]"""
+    """Optional array of shape `(n_sites,)` used to restrict the convolutional
+        kernel. Only parameters with mask :math:'\ne 0' are used. For best performance a
+        boolean mask should be used"""
     param_dtype: DType = jnp.float64
     """The dtype of the weights."""
     precision: Any = None
@@ -180,7 +181,7 @@ class DenseSymmFFT(Module):
         self.sites_per_cell = sg.shape[1] // self.n_cells
 
         if self.mask is not None:
-            self.scaled_mask = _normalise_mask(self.mask, sg.shape[1])
+            (self.kernel_indices,) = np.nonzero(self.mask.wrapped)
 
         # maps (n_sites) dimension of kernels to (sites_per_cell, n_point, *shape)
         # as used in FFT-based group convolution
@@ -213,11 +214,6 @@ class DenseSymmFFT(Module):
         x = x.transpose(0, 1, 3, 2)
         x = x.reshape(*x.shape[:-1], *self.shape)
 
-        if self.mask is not None:
-            scaled_mask = self.scaled_mask
-        else:
-            scaled_mask = None
-
         if self.use_bias:
             bias = self.param(
                 "bias", self.bias_init, (self.features,), self.param_dtype
@@ -225,20 +221,29 @@ class DenseSymmFFT(Module):
         else:
             bias = None
 
-        kernel = self.param(
-            "kernel",
-            self.kernel_init,
-            (self.features, in_features, self.n_cells * self.sites_per_cell),
-            self.param_dtype,
-        )
-
-        x, scaled_mask, kernel, bias = promote_dtype(
-            x, scaled_mask, kernel, bias, dtype=None
-        )
-        dtype = x.dtype
-
         if self.mask is not None:
-            kernel = kernel * jnp.expand_dims(scaled_mask, (0, 1))
+            kernel_params = self.param(
+                "kernel",
+                self.kernel_init,
+                (self.features, in_features, len(self.kernel_indices)),
+                self.param_dtype,
+            )
+
+            kernel = jnp.zeros(
+                [self.features, in_features, self.n_cells * self.sites_per_cell],
+                self.param_dtype,
+            )
+            kernel = kernel.at[:, :, self.kernel_indices].set(kernel_params)
+        else:
+            kernel = self.param(
+                "kernel",
+                self.kernel_init,
+                (self.features, in_features, self.n_cells * self.sites_per_cell),
+                self.param_dtype,
+            )
+
+        x, kernel, bias = promote_dtype(x, kernel, bias, dtype=None)
+        dtype = x.dtype
 
         # Converts the convolutional kernel of shape (features, in_features, n_sites)
         # to the expanded kernel of shape (features, in_features, sites_per_cell,
@@ -287,7 +292,9 @@ class DenseEquivariantFFT(Module):
     use_bias: bool = True
     """Whether to add a bias to the output (default: True)."""
     mask: Optional[HashableArray] = None
-    """Mask that zeros out elements of the filter. Should be of shape [inp.shape[-1]]"""
+    """Optional array of shape `(n_symm,)` where `(n_symm,)` = `len(graph.automorphisms())`
+        used to restrict the convolutional kernel. Only parameters with mask :math:'\ne 0' are used.
+        For best performance a boolean mask should be used"""
     param_dtype: DType = jnp.float64
     """The dtype of the weights."""
     precision: Any = None
@@ -305,7 +312,7 @@ class DenseEquivariantFFT(Module):
         self.n_cells = np.product(np.asarray(self.shape))
         self.n_point = len(pt) // self.n_cells
         if self.mask is not None:
-            self.scaled_mask = _normalise_mask(self.mask, len(pt))
+            (self.kernel_indices,) = np.nonzero(self.mask.wrapped)
 
         # maps (n_sites) dimension of kernels to (n_point, n_point, *shape)
         # as used in FFT-based group convolution
@@ -334,15 +341,26 @@ class DenseEquivariantFFT(Module):
         else:
             bias = None
 
-        kernel = self.param(
-            "kernel",
-            self.kernel_init,
-            (self.features, in_features, self.n_point * self.n_cells),
-            self.param_dtype,
-        )
-
         if self.mask is not None:
-            kernel = kernel * jnp.expand_dims(self.scaled_mask, (0, 1))
+            kernel_params = self.param(
+                "kernel",
+                self.kernel_init,
+                (self.features, in_features, len(self.kernel_indices)),
+                self.param_dtype,
+            )
+
+            kernel = jnp.zeros(
+                [self.features, in_features, self.n_point * self.n_cells],
+                self.param_dtype,
+            )
+            kernel = kernel.at[:, :, self.kernel_indices].set(kernel_params)
+        else:
+            kernel = self.param(
+                "kernel",
+                self.kernel_init,
+                (self.features, in_features, self.n_point * self.n_cells),
+                self.param_dtype,
+            )
 
         x, kernel, bias = promote_dtype(x, kernel, bias, dtype=None)
         dtype = x.dtype
@@ -412,7 +430,9 @@ class DenseEquivariantIrrep(Module):
     use_bias: bool = True
     """Whether to add a bias to the output (default: True)."""
     mask: Optional[HashableArray] = None
-    """Mask that zeros out elements of the filter. Should be of shape [inp.shape[-1]]"""
+    """Optional array of shape `(n_symm,)` where `(n_symm,)` = `len(graph.automorphisms())`
+        used to restrict the convolutional kernel. Only parameters with mask :math:'\ne 0' are used.
+        For best performance a boolean mask should be used"""
 
     param_dtype: DType = jnp.float64
     """The dtype of the weights."""
@@ -427,7 +447,7 @@ class DenseEquivariantIrrep(Module):
     def setup(self):
         self.n_symm = self.irreps[0].shape[0]
         if self.mask is not None:
-            self.scaled_mask = _normalise_mask(self.mask, self.n_symm)
+            (self.kernel_indices,) = np.nonzero(self.mask.wrapped)
 
         self.forward = jnp.concatenate(
             [jnp.asarray(irrep).reshape(self.n_symm, -1) for irrep in self.irreps],
@@ -516,15 +536,25 @@ class DenseEquivariantIrrep(Module):
         else:
             bias = None
 
-        kernel = self.param(
-            "kernel",
-            self.kernel_init,
-            (self.features, in_features, self.n_symm),
-            self.param_dtype,
-        )
-
         if self.mask is not None:
-            kernel = kernel * jnp.expand_dims(self.scaled_mask, (0, 1))
+            kernel_params = self.param(
+                "kernel",
+                self.kernel_init,
+                (self.features, in_features, len(self.kernel_indices)),
+                self.param_dtype,
+            )
+
+            kernel = jnp.zeros(
+                [self.features, in_features, self.n_symm], self.param_dtype
+            )
+            kernel = kernel.at[:, :, self.kernel_indices].set(kernel_params)
+        else:
+            kernel = self.param(
+                "kernel",
+                self.kernel_init,
+                (self.features, in_features, self.n_symm),
+                self.param_dtype,
+            )
 
         x, kernel, bias = promote_dtype(x, kernel, bias, dtype=None)
         dtype = x.dtype
@@ -565,7 +595,9 @@ class DenseEquivariantMatrix(Module):
     use_bias: bool = True
     """Whether to add a bias to the output (default: True)."""
     mask: Optional[HashableArray] = None
-    """Whether to mask the filters to restrict the connectivity"""
+    """Optional array of shape `(n_symm,)` where `(n_symm,)` = `len(graph.automorphisms())`
+        used to restrict the convolutional kernel. Only parameters with mask :math:'\ne 0' are used.
+        For best performance a boolean mask should be used"""
     param_dtype: Any = jnp.float64
     """The dtype of the weights."""
     precision: Any = None
@@ -579,7 +611,7 @@ class DenseEquivariantMatrix(Module):
     def setup(self):
         self.n_symm = np.asarray(self.product_table).shape[0]
         if self.mask is not None:
-            self.scaled_mask = _normalise_mask(self.mask, self.n_symm)
+            (self.kernel_indices,) = np.nonzero(self.mask.wrapped)
 
     @compact
     def __call__(self, x: Array) -> Array:
@@ -591,12 +623,25 @@ class DenseEquivariantMatrix(Module):
         """
         in_features = x.shape[-2]
 
-        kernel = self.param(
-            "kernel",
-            self.kernel_init,
-            (self.features, in_features, self.n_symm),
-            self.param_dtype,
-        )
+        if self.mask is not None:
+            kernel_params = self.param(
+                "kernel",
+                self.kernel_init,
+                (self.features, in_features, len(self.kernel_indices)),
+                self.param_dtype,
+            )
+
+            kernel = jnp.zeros(
+                [self.features, in_features, self.n_symm], self.param_dtype
+            )
+            kernel = kernel.at[:, :, self.kernel_indices].set(kernel_params)
+        else:
+            kernel = self.param(
+                "kernel",
+                self.kernel_init,
+                (self.features, in_features, self.n_symm),
+                self.param_dtype,
+            )
 
         if self.use_bias:
             bias = self.param(
@@ -605,9 +650,6 @@ class DenseEquivariantMatrix(Module):
         else:
             bias = None
 
-        if self.mask is not None:
-            kernel = kernel * jnp.expand_dims(self.scaled_mask, (0, 1))
-
         kernel, bias, x = promote_dtype(kernel, bias, x, dtype=None)
 
         # Converts the convolutional kernel of shape (features, in_features, n_symm)
@@ -632,7 +674,9 @@ class DenseEquivariantMatrix(Module):
 
 
 @deprecate_dtype
-def DenseSymm(symmetries, point_group=None, mode="auto", shape=None, **kwargs):
+def DenseSymm(
+    symmetries, point_group=None, mode="auto", shape=None, mask=None, **kwargs
+):
     r"""
     Implements a projection onto a symmetry group. The output will be
     equivariant with respect to the symmetry operations in the group and can
@@ -661,8 +705,9 @@ def DenseSymm(symmetries, point_group=None, mode="auto", shape=None, **kwargs):
         features: The number of output features. The full output shape
             is :code:`[n_batch,features,n_symm]`.
         use_bias: A bool specifying whether to add a bias to the output (default: True).
-        mask: An optional array of shape [n_sites] consisting of ones and zeros
-            that can be used to give the kernel a particular shape.
+        mask: Optional array of shape `(n_sites,)` used to restrict the convolutional
+        kernel. Only parameters with mask :math:'\ne 0' are used. For best performance a
+        boolean mask should be used.
         param_dtype: The datatype of the weights. Defaults to a 64bit float.
         precision: Optional argument specifying numerical precision of the computation.
             see {class}`jax.lax.Precision` for details.
@@ -670,6 +715,9 @@ def DenseSymm(symmetries, point_group=None, mode="auto", shape=None, **kwargs):
         bias_init: Optional bias initialization function. Defaults to zero initialization.
 
     """
+    if mask is not None:
+        mask = HashableArray(mask)
+
     if isinstance(symmetries, Lattice) and (
         point_group is not None or symmetries._point_group is not None
     ):
@@ -697,9 +745,9 @@ def DenseSymm(symmetries, point_group=None, mode="auto", shape=None, **kwargs):
                 "the symmetries keyword argument."
             )
         else:
-            return DenseSymmFFT(sym, shape=shape, **kwargs)
+            return DenseSymmFFT(sym, shape=shape, mask=mask, **kwargs)
     elif mode in ["matrix", "auto"]:
-        return DenseSymmMatrix(sym, **kwargs)
+        return DenseSymmMatrix(sym, mask=mask, **kwargs)
     else:
         raise ValueError(
             f"Unknown mode={mode}. Valid modes are 'fft', 'matrix', or 'auto'."
@@ -714,6 +762,7 @@ def DenseEquivariant(
     shape=None,
     point_group=None,
     in_features=None,
+    mask=None,
     **kwargs,
 ):
     r"""A group convolution operation that is equivariant over a symmetry group.
@@ -749,14 +798,18 @@ def DenseEquivariant(
         features: The number of output features. The full output shape
             is [n_batch,features,n_symm].
         use_bias: A bool specifying whether to add a bias to the output (default: True).
-        mask: An optional array of shape [n_sites] consisting of ones and zeros
-            that can be used to give the kernel a particular shape.
+        mask: Optional array of shape `(n_symm,)` where `(n_symm,)` = `len(graph.automorphisms())`
+        used to restrict the convolutional kernel. Only parameters with mask :math:'\ne 0' are used.
+        For best performance a boolean mask should be used.
         param_dtype: The datatype of the weights. Defaults to a 64bit float.
         precision: Optional argument specifying numerical precision of the computation.
             see :class:`jax.lax.Precision` for details.
         kernel_init: Optional kernel initialization function. Defaults to variance scaling.
         bias_init: Optional bias initialization function. Defaults to zero initialization.
     """
+    if mask is not None:
+        mask = HashableArray(mask)
+
     # deprecate in_features
     if in_features is not None:
         warn_deprecation(
@@ -807,13 +860,13 @@ def DenseEquivariant(
     elif isinstance(symmetries, Sequence):
         if mode not in ["irreps", "auto"]:
             raise ValueError("Specification of symmetries incompatible with mode")
-        return DenseEquivariantIrrep(symmetries, **kwargs)
+        return DenseEquivariantIrrep(symmetries, mask=mask, **kwargs)
     else:
         if symmetries.ndim == 2 and symmetries.shape[0] == symmetries.shape[1]:
             if mode == "irreps":
                 raise ValueError("Specification of symmetries incompatible with mode")
             elif mode == "matrix":
-                return DenseEquivariantMatrix(symmetries, **kwargs)
+                return DenseEquivariantMatrix(symmetries, mask=mask, **kwargs)
             else:
                 if shape is None:
                     raise TypeError(
@@ -822,7 +875,9 @@ def DenseEquivariant(
                         "the symmetries keyword argument."
                     )
                 else:
-                    return DenseEquivariantFFT(symmetries, shape=shape, **kwargs)
+                    return DenseEquivariantFFT(
+                        symmetries, mask=mask, shape=shape, **kwargs
+                    )
         return ValueError("Invalid Specification of Symmetries")
 
     if mode == "fft":
@@ -834,13 +889,15 @@ def DenseEquivariant(
             )
         else:
             return DenseEquivariantFFT(
-                HashableArray(sg.product_table), shape=shape, **kwargs
+                HashableArray(sg.product_table), mask=mask, shape=shape, **kwargs
             )
     elif mode in ["irreps", "auto"]:
         irreps = tuple(HashableArray(irrep) for irrep in sg.irrep_matrices())
-        return DenseEquivariantIrrep(irreps, **kwargs)
+        return DenseEquivariantIrrep(irreps, mask=mask, **kwargs)
     elif mode == "matrix":
-        return DenseEquivariantMatrix(HashableArray(sg.product_table), **kwargs)
+        return DenseEquivariantMatrix(
+            HashableArray(sg.product_table), mask=mask, **kwargs
+        )
     else:
         raise ValueError(
             f"Unknown mode={mode}. Valid modes are 'fft', 'matrix', 'irreps' or 'auto'."
