@@ -15,7 +15,6 @@
 import abc
 
 from netket.utils.types import DType
-from netket.utils import deprecated
 
 from netket.hilbert import AbstractHilbert
 
@@ -36,13 +35,6 @@ class AbstractOperator(abc.ABC):
         r"""The hilbert space associated to this operator."""
         return self._hilbert
 
-    # TODO: eventually remove this
-    @property
-    @deprecated(reason="Please use `operator.hilbert.size` instead.")
-    def size(self) -> int:
-        r"""The total number number of local degrees of freedom."""
-        return self._hilbert.size
-
     @property
     def is_hermitian(self) -> bool:
         """Returns true if this operator is hermitian."""
