@@ -20,15 +20,16 @@ from textwrap import dedent
 
 import numpy as np
 import numba
+from scipy.sparse import issparse
 
 from netket.hilbert import AbstractHilbert
 from netket.utils.types import DType, Array
+from netket.utils.numbers import dtype as _dtype
 
 from ._discrete_operator import DiscreteOperator
 from ._lazy import Transpose
 
 from ._local_operator_helpers import (
-    _dtype,
     canonicalize_input,
     _multiply_operators,
     cast_operator_matrix_dtype,
@@ -37,7 +38,10 @@ from ._local_operator_compile_helpers import pack_internals
 
 
 def is_hermitian(a: np.ndarray, rtol=1e-05, atol=1e-08) -> bool:
-    return np.allclose(a, a.T.conj(), rtol=rtol, atol=atol)
+    if issparse(a):
+        return np.allclose(a.todense(), a.T.conj().todense(), rtol=rtol, atol=atol)
+    else:
+        return np.allclose(a, a.T.conj(), rtol=rtol, atol=atol)
 
 
 def _is_sorted(a):
@@ -66,10 +70,15 @@ class LocalOperator(DiscreteOperator):
         specified) a constant level shift.
 
         Args:
-           hilbert (netket.AbstractHilbert): Hilbert space the operator acts on.
-           operators (list(numpy.array) or numpy.array): A list of operators, in matrix form.
-           acting_on (list(numpy.array) or numpy.array): A list of sites, which the corresponding operators act on.
-           constant (float): Level shift for operator. Default is 0.0.
+           hilbert: Hilbert space the operator acts on.
+           operators: A list of operators, in matrix form. Supports numpy dense or scipy
+           sparse format
+           acting_on: A list of list of sites, which the corresponding operators act on. This
+                should be constructed such that :code:`operators[i]` acts on the sites :code:`acting_on[i]`.
+                If operators is not a list of operators, acting_on should just be the list of
+                corresponding sites.
+           constant: Constant diagonal shift of the operator, equivalent to
+                :math:`+\text{c}\hat{I}`. Default is 0.0.
 
         Examples:
            Constructs a ``LocalOperator`` without any operators.
@@ -84,6 +93,7 @@ class LocalOperator(DiscreteOperator):
         super().__init__(hilbert)
         self.mel_cutoff = 1.0e-6
         self._initialized = None
+        self._is_hermitian = None
 
         if not all(
             [_is_sorted(hilbert.states_at_index(i)) for i in range(hilbert.size)]
@@ -160,7 +170,12 @@ class LocalOperator(DiscreteOperator):
         # TODO: (VolodyaCO) I guess that if we have an operator with diagonal elements equal to 1j*C+Y, some complex constant, and
         # self._constant=-1j*C, then the actual diagonal would be Y. How do we check hermiticity taking into account the diagonal
         # elements as well as the self._constant? For the moment I just check hermiticity of the added constant, which must be real.
-        return all(map(is_hermitian, self.operators)) and np.isreal(self._constant)
+        if self._is_hermitian is None:
+            self._is_hermitian = all(map(is_hermitian, self.operators)) and np.isreal(
+                self._constant
+            )
+
+        return self._is_hermitian
 
     @property
     def mel_cutoff(self) -> float:
@@ -363,7 +378,14 @@ class LocalOperator(DiscreteOperator):
                     )
                 )
 
+        is_hermitian_A = self._is_hermitian
+        is_hermitian_B = other._is_hermitian
+
         self._reset_caches()
+
+        if is_hermitian_A and is_hermitian_B:
+            self._is_hermitian = is_hermitian_A
+
         return self
 
     def _reset_caches(self):
@@ -371,6 +393,7 @@ class LocalOperator(DiscreteOperator):
         Cleans the internal caches built on the operator.
         """
         self._initialized = False
+        self._is_hermitian = None
 
     def _setup(self, force: bool = False):
         """Analyze the operator strings and precompute arrays for get_conn inference"""
