@@ -12,36 +12,22 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from typing import Tuple, Union
+from typing import Tuple
 
 import functools
 import numbers
 
 import numpy as np
 
+from scipy import sparse
 from scipy.sparse import spmatrix
 
 from netket.hilbert import AbstractHilbert, Fock
 from netket.utils.types import DType, Array
-
-from ._discrete_operator import AbstractOperator
+from netket.utils.numbers import dtype as _dtype
 
 
-def _dtype(obj: Union[numbers.Number, Array, AbstractOperator]) -> DType:
-    """
-    Returns the dtype of the input object
-    """
-    if isinstance(obj, numbers.Number):
-        return type(obj)
-    elif isinstance(obj, AbstractOperator):
-        return obj.dtype
-    elif isinstance(obj, np.ndarray):
-        return obj.dtype
-    else:
-        raise TypeError(f"cannot deduce dtype of object type {type(obj)}: {obj}")
-
-
-def cast_operator_matrix_dtype(matrix, dtype):
+def cast_operator_matrix_dtype(matrix: Array, dtype: DType):
     """
     Changes the dtype of a matrix, without changing the structural type of the object.
 
@@ -61,12 +47,27 @@ def _standardize_matrix_input_type(op):
     """
     if isinstance(op, list):
         return np.asarray(op)
-    elif isinstance(op, spmatrix):
-        return op.todense()
+    elif sparse.issparse(op):
+        return op.tocoo()
     else:
         return op
 
 
+def _kron(A, B):
+    if isinstance(A, spmatrix) and isinstance(B, spmatrix):
+        res = sparse.kron(A, B, format="coo")
+    else:
+        res = np.kron(np.asarray(A), np.asarray(B))
+    return res
+
+
+def _eye_like(size, *, dtype, like):
+    if sparse.issparse(like):
+        return sparse.eye(size, dtype=dtype)
+    else:
+        return np.eye(size, dtype=dtype)
+
+
 def canonicalize_input(
     hilbert: AbstractHilbert, operators, acting_on, constant, *, dtype=None
 ):
@@ -100,6 +101,12 @@ def canonicalize_input(
         if max(map(max, acting_on)) >= hilbert.size or min(map(min, acting_on)) < 0:
             raise ValueError("An operator acts on an invalid set of sites.")
 
+        for i, aon in enumerate(acting_on):
+            if len(aon) != len(set(aon)):
+                raise ValueError(
+                    f"The operator at index {i} acts on duplicated sites {aon}"
+                )
+
     acting_on = [tuple(aon) for aon in acting_on]
     # operators = [np.asarray(operator) for operator in operators]
     operators = [_standardize_matrix_input_type(op) for op in operators]
@@ -193,7 +200,17 @@ def _reorder_kronecker_product(hi, mat, acting_on) -> Tuple[Array, Tuple]:
     n_unsorted = hi_unsorted_subspace.states_to_numbers(v_unsorted)
 
     # reorder the matrix
-    mat_sorted = mat[n_unsorted, :][:, n_unsorted]
+    if sparse.issparse(mat):
+        # covert the indices from going from old -> new to new->old
+        idxs = np.argsort(n_unsorted)
+
+        # sparse matrices don't support indexing, and can be sorted more
+        # quickly
+        mat_sorted = mat.copy()
+        mat_sorted.row[:] = idxs[mat.row]
+        mat_sorted.col[:] = idxs[mat.col]
+    else:
+        mat_sorted = mat[n_unsorted, :][:, n_unsorted]
 
     return mat_sorted, tuple(acting_on_sorted)
 
@@ -216,7 +233,7 @@ def _multiply_operators(
     elif inters.size == 0:
         # disjoint supports
         support = tuple(np.concatenate([support_A, support_B]))
-        operator = np.kron(A, B)
+        operator = _kron(A, B)
         operator, support = _reorder_kronecker_product(hilbert, operator, support)
         return tuple(support), operator
     else:
@@ -229,24 +246,24 @@ def _multiply_operators(
         supp_B_min = min(support_B)
         for site in support_A:
             if site not in support_B:
-                I = np.eye(hilbert.shape[site], dtype=dtype)
+                I = _eye_like(hilbert.shape[site], dtype=dtype, like=B)
                 if site < supp_B_min:
                     _support_B = [site] + _support_B
-                    _B = np.kron(I, _B)
+                    _B = _kron(I, _B)
                 else:  # site > actmax
                     _support_B = _support_B + [site]
-                    _B = np.kron(_B, I)
+                    _B = _kron(_B, I)
 
         supp_A_min = min(support_A)
         for site in support_B:
             if site not in support_A:
-                I = np.eye(hilbert.shape[site], dtype=dtype)
+                I = _eye_like(hilbert.shape[site], dtype=dtype, like=A)
                 if site < supp_A_min:
                     _support_A = [site] + _support_A
-                    _A = np.kron(I, _A)
+                    _A = _kron(I, _A)
                 else:  # site > actmax
                     _support_A = _support_A + [site]
-                    _A = np.kron(_A, I)
+                    _A = _kron(_A, I)
 
         # reorder
         _A, _support_A = _reorder_kronecker_product(hilbert, _A, _support_A)
