@@ -21,7 +21,9 @@ from numba import jit
 from itertools import product
 
 from netket.hilbert import Qubit, AbstractHilbert
+from netket.utils.numbers import is_scalar
 
+from ._abstract_operator import AbstractOperator
 from ._discrete_operator import DiscreteOperator
 
 valid_pauli_regex = re.compile(r"^[XYZI]+$")
@@ -33,8 +35,8 @@ class PauliStrings(DiscreteOperator):
     def __init__(
         self,
         hilbert: AbstractHilbert,
-        operators: List[str] = None,
-        weights: List[Union[float, complex]] = None,
+        operators: Union[str, List[str]] = None,
+        weights: Union[float, complex, List[Union[float, complex]]] = None,
         *,
         cutoff: float = 1.0e-10,
         dtype: DType = complex,
@@ -77,12 +79,19 @@ class PauliStrings(DiscreteOperator):
                 "None valued operators passed. (Might arise when passing None valued hilbert explicitly)"
             )
 
+        # Support single-operator
+        if isinstance(operators, str):
+            operators = [operators]
+
         if len(operators) == 0:
             raise ValueError("No Pauli operators passed.")
 
+        # default weight is 1
         if weights is None:
-            # default weight is 1
-            weights = [True for i in operators]
+            weights = True
+
+        if is_scalar(weights):
+            weights = [weights for _ in operators]
 
         if len(weights) != len(operators):
             raise ValueError("weights should have the same length as operators.")
@@ -105,11 +114,11 @@ class PauliStrings(DiscreteOperator):
         if hilbert is None:
             hilbert = Qubit(_hilb_size)
 
-        super().__init__(hilbert)
-        if self.hilbert.local_size != 2:
+        if not np.allclose(hilbert.shape, 2):
             raise ValueError(
                 "PauliStrings only work for local hilbert size 2 where PauliMatrices are defined"
             )
+        super().__init__(hilbert)
 
         self._cutoff = cutoff
         b_weights = np.asarray(weights, dtype=dtype)
@@ -123,9 +132,7 @@ class PauliStrings(DiscreteOperator):
 
     @staticmethod
     def identity(hilbert: AbstractHilbert, **kwargs):
-        operators = ("I" * hilbert.size,)
-        weights = (1.0,)
-        return PauliStrings(hilbert, operators, weights, **kwargs)
+        return PauliStrings(hilbert, "I" * hilbert.size, **kwargs)
 
     def _setup(self, force=False):
         """Analyze the operator strings and precompute arrays for get_conn inference"""
@@ -212,7 +219,13 @@ class PauliStrings(DiscreteOperator):
             self._mels_max = np.empty((n_operators), dtype=dtype)
             self._n_operators = n_operators
 
-            self._local_states = np.array(self.hilbert.local_states)
+            self._local_states = np.array(self.hilbert.states_at_index(0))
+            for i in range(1, self.hilbert.size):
+                if not np.allclose(self._local_states, self.hilbert.states_at_index(i)):
+                    raise ValueError(
+                        "Hilbert spaces with non homogeneous local_states are not "
+                        "yet supported."
+                    )
 
             self._initialized = True
 
@@ -323,6 +336,16 @@ class PauliStrings(DiscreteOperator):
         return self * scalar
 
     def __mul__(self, scalar):
+        if isinstance(scalar, AbstractOperator):
+            raise TypeError(
+                "To multiply operators use the matrix`@` "
+                "multiplication operator `@` instead of the element-wise "
+                "multiplication operator `*`.\n\n"
+                "For example:\n\n"
+                ">>> nk.operator.PauliStrings('XY')@nk.operator.PauliStrings('ZY')"
+                "\n\n"
+            )
+
         if not np.issubdtype(type(scalar), np.number):
             raise NotImplementedError
         weights = self._orig_weights * scalar
