@@ -19,15 +19,20 @@ import jax
 from jax import numpy as jnp
 from flax import struct
 
-from netket.utils.types import PyTree
+from netket.utils.types import Scalar, PyTree
 from netket.utils import mpi
 import netket.jax as nkjax
+from netket.nn import split_array_mpi
 
 from ..linear_operator import LinearOperator, Uninitialized
 
 from .common import check_valid_vector_type
-from .qgt_jacobian_dense_logic import prepare_centered_oks, vec_to_real
-from .qgt_jacobian_common import choose_jacobian_mode
+from .qgt_jacobian_common import (
+    choose_jacobian_mode,
+    sanitize_diag_shift,
+    to_shift_offset,
+    rescale,
+)
 
 
 def QGTJacobianDense(
@@ -35,7 +40,10 @@ def QGTJacobianDense(
     *,
     mode: str = None,
     holomorphic: bool = None,
-    rescale_shift=False,
+    diag_shift=None,
+    diag_scale=None,
+    rescale_shift=None,
+    chunk_size=None,
     **kwargs,
 ) -> "QGTJacobianDenseT":
     """
@@ -47,6 +55,18 @@ def QGTJacobianDense(
     The details on how the ⟨S⟩⁻¹⟨F⟩ system is solved are contained in
     the field `sr`.
 
+    Numerical estimates of the QGT are usually ill-conditioned and require
+    regularisation. The standard approach is to add a positive constant to the diagonal;
+    alternatively, Becca and Sorella (2017) propose scaling this offset with the
+    diagonal entry itself. NetKet allows using both in tandem:
+
+    .. math::
+
+        S_{ii} \\mapsto S_{ii} + \\epsilon_1 S_{ii} + \\epsilon_2;
+
+    :math:`\\epsilon_{1,2}` are specified using `diag_scale` and `diag_shift`,
+    respectively.
+
     Args:
         vstate: The variational state
         mode: "real", "complex" or "holomorphic": specifies the implementation
@@ -56,57 +76,84 @@ def QGTJacobianDense(
               models. holomorphic works for any function assuming it's holomorphic
               or real valued.
         holomorphic: a flag to indicate that the function is holomorphic.
-        rescale_shift: If True rescales the diagonal shift
+        diag_scale: Fractional shift :math:`\\epsilon_1` added to diagonal entries (see above).
+        diag_shift: Constant shift :math:`\\epsilon_2` added to diagonal entries (see above).
+        chunk_size: If supplied, overrides the chunk size of the variational state
+                    (useful for models where the backward pass requires more
+                    memory than the forward pass).
     """
+    if mode is not None and holomorphic is not None:
+        raise ValueError("Cannot specify both `mode` and `holomorphic`.")
+    if rescale_shift is not None and diag_scale is not None:
+        raise ValueError("Cannot specify both `rescale_shift` and `diag_scale`.")
 
     if vstate is None:
         return partial(
             QGTJacobianDense,
             mode=mode,
             holomorphic=holomorphic,
+            chunk_size=chunk_size,
+            diag_shift=diag_shift,
+            diag_scale=diag_scale,
             rescale_shift=rescale_shift,
             **kwargs,
         )
 
+    diag_shift, diag_scale = sanitize_diag_shift(diag_shift, diag_scale, rescale_shift)
+
     # TODO: Find a better way to handle this case
     from netket.vqs import ExactState
 
     if isinstance(vstate, ExactState):
-        raise TypeError("Only QGTJacobianPyTree works with ExactState.")
+        samples = split_array_mpi(vstate._all_states)
+        pdf = split_array_mpi(vstate.probability_distribution())
+    else:
+        samples = vstate.samples
+        pdf = None
 
     if mode is None:
         mode = choose_jacobian_mode(
             vstate._apply_fun,
             vstate.parameters,
             vstate.model_state,
-            vstate.samples,
-            mode=mode,
+            samples,
             holomorphic=holomorphic,
         )
-    elif holomorphic is not None:
-        raise ValueError("Cannot specify both `mode` and `holomorphic`.")
 
-    if hasattr(vstate, "chunk_size"):
+    if chunk_size is None and hasattr(vstate, "chunk_size"):
         chunk_size = vstate.chunk_size
-    else:
-        chunk_size = None
 
-    O, scale = prepare_centered_oks(
+    shift, offset = to_shift_offset(diag_shift, diag_scale)
+
+    jacobians = nkjax.jacobian(
         vstate._apply_fun,
         vstate.parameters,
-        vstate.samples.reshape(-1, vstate.samples.shape[-1]),
+        samples.reshape(-1, samples.shape[-1]),
         vstate.model_state,
-        mode,
-        rescale_shift,
-        chunk_size,
+        mode=mode,
+        pdf=pdf,
+        chunk_size=chunk_size,
+        dense=True,
+        center=True,
     )
 
+    if offset is not None:
+        ndims = 1 if mode != "complex" else 2
+        jacobians, scale = rescale(jacobians, offset, ndims=ndims)
+    else:
+        scale = None
+
     pars_struct = jax.tree_map(
         lambda x: jax.ShapeDtypeStruct(x.shape, x.dtype), vstate.parameters
     )
 
     return QGTJacobianDenseT(
-        O=O, scale=scale, mode=mode, _params_structure=pars_struct, **kwargs
+        O=jacobians,
+        scale=scale,
+        mode=mode,
+        _params_structure=pars_struct,
+        diag_shift=shift,
+        **kwargs,
     )
 
 
@@ -150,12 +197,52 @@ class QGTJacobianDenseT(LinearOperator):
 
     _params_structure: PyTree = struct.field(pytree_node=False, default=Uninitialized)
 
+    @jax.jit
     def __matmul__(self, vec: Union[PyTree, jnp.ndarray]) -> Union[PyTree, jnp.ndarray]:
-        return _matmul(self, vec)
+        if not hasattr(vec, "ndim") and not self._in_solve:
+            check_valid_vector_type(self._params_structure, vec)
 
+        vec, reassemble = convert_tree_to_dense_format(
+            vec, self.mode, disable=self._in_solve
+        )
+
+        if self.scale is not None:
+            vec = vec * self.scale
+
+        result = mat_vec(vec, self.O, self.diag_shift)
+
+        if self.scale is not None:
+            result = result * self.scale
+
+        return reassemble(result)
+
+    @jax.jit
     def _solve(self, solve_fun, y: PyTree, *, x0: Optional[PyTree] = None) -> PyTree:
-        return _solve(self, solve_fun, y, x0=x0)
+        if not hasattr(y, "ndim"):
+            check_valid_vector_type(self._params_structure, y)
 
+        y, reassemble = convert_tree_to_dense_format(y, self.mode)
+
+        if x0 is not None:
+            x0, _ = convert_tree_to_dense_format(x0, self.mode)
+            if self.scale is not None:
+                x0 = x0 * self.scale
+
+        if self.scale is not None:
+            y = y / self.scale
+
+        # to pass the object LinearOperator itself down
+        # but avoid rescaling, we pass down an object with
+        # scale = None
+        unscaled_self = self.replace(scale=None, _in_solve=True)
+        out, info = solve_fun(unscaled_self, y, x0=x0)
+
+        if self.scale is not None:
+            out = out / self.scale
+
+        return reassemble(out), info
+
+    @jax.jit
     def to_dense(self) -> jnp.ndarray:
         """
         Convert the lazy matrix representation to a dense matrix representation.
@@ -163,7 +250,16 @@ class QGTJacobianDenseT(LinearOperator):
         Returns:
             A dense matrix representation of this S matrix.
         """
-        return _to_dense(self)
+        if self.scale is None:
+            O = self.O
+            diag = jnp.eye(self.O.shape[-1])
+        else:
+            O = self.O * self.scale[jnp.newaxis, :]
+            diag = jnp.diag(self.scale**2)
+
+        # concatenate samples with real/Imaginary dimension
+        O = O.reshape(-1, O.shape[-1])
+        return mpi.mpi_sum_jax(O.conj().T @ O)[0] + self.diag_shift * diag
 
     def __repr__(self):
         return (
@@ -172,93 +268,34 @@ class QGTJacobianDenseT(LinearOperator):
         )
 
 
-########################################################################################
-#####                                  QGT Logic                                   #####
-########################################################################################
+#################################################
+#####           QGT internal Logic          #####
+#################################################
 
 
-@jax.jit
-def _matmul(
-    self: QGTJacobianDenseT, vec: Union[PyTree, jnp.ndarray]
-) -> Union[PyTree, jnp.ndarray]:
-
-    unravel = None
-    if not hasattr(vec, "ndim") and not self._in_solve:
-        check_valid_vector_type(self._params_structure, vec)
-        vec, unravel = nkjax.tree_ravel(vec)
-
-    # Real-imaginary split RHS in R→R and R→C modes
-    reassemble = None
-    if self.mode != "holomorphic" and not self._in_solve:
-        vec, reassemble = vec_to_real(vec)
-
-    if self.scale is not None:
-        vec = vec * self.scale
-
-    result = (
-        mpi.mpi_sum_jax(((self.O @ vec).T.conj() @ self.O).T.conj())[0]
-        + self.diag_shift * vec
-    )
-
-    if self.scale is not None:
-        result = result * self.scale
-
-    if reassemble is not None:
-        result = reassemble(result)
-
-    if unravel is not None:
-        result = unravel(result)
-
-    return result
+def mat_vec(v: PyTree, O: PyTree, diag_shift: Scalar) -> PyTree:
+    w = O @ v
+    res = jnp.tensordot(w.conj(), O, axes=w.ndim).conj()
+    return mpi.mpi_sum_jax(res)[0] + diag_shift * v
 
 
-@jax.jit
-def _solve(
-    self: QGTJacobianDenseT, solve_fun, y: PyTree, *, x0: Optional[PyTree] = None
-) -> PyTree:
-    if not hasattr(y, "ndim"):
-        check_valid_vector_type(self._params_structure, y)
+def convert_tree_to_dense_format(vec, mode, *, disable=False):
+    """
+    Converts an arbitrary PyTree/vector which might be real/complex
+    to the dense-(maybe-real)-vector used for QGTJacobian.
 
-    # Ravel input PyTrees, record unravelling function too
-    y, unravel = nkjax.tree_ravel(y)
+    The format is dictated by the sequence of operations chosen by
+    `nk.jax.jacobian(..., dense=True)`. As `nk.jax.jacobian` first
+    converts the pytree of parameters to real and then concatenates
+    real and imaginary terms with a tree_ravel, we must do the same
+    in here.
+    """
+    unravel = lambda x: x
+    reassemble = lambda x: x
+    if not disable:
+        if mode != "holomorphic":
+            vec, reassemble = nkjax.tree_to_real(vec)
+        if not hasattr(vec, "ndim"):
+            vec, unravel = nkjax.tree_ravel(vec)
 
-    if self.mode != "holomorphic":
-        y, reassemble = vec_to_real(y)
-
-    if x0 is not None:
-        x0, _ = nkjax.tree_ravel(x0)
-        if self.mode != "holomorphic":
-            x0, _ = vec_to_real(x0)
-
-        if self.scale is not None:
-            x0 = x0 * self.scale
-
-    if self.scale is not None:
-        y = y / self.scale
-
-    # to pass the object LinearOperator itself down
-    # but avoid rescaling, we pass down an object with
-    # scale = None
-    unscaled_self = self.replace(scale=None, _in_solve=True)
-
-    out, info = solve_fun(unscaled_self, y, x0=x0)
-
-    if self.scale is not None:
-        out = out / self.scale
-
-    if self.mode != "holomorphic":
-        out = reassemble(out)
-
-    return unravel(out), info
-
-
-@jax.jit
-def _to_dense(self: QGTJacobianDenseT) -> jnp.ndarray:
-    if self.scale is None:
-        O = self.O
-        diag = jnp.eye(self.O.shape[1])
-    else:
-        O = self.O * self.scale[jnp.newaxis, :]
-        diag = jnp.diag(self.scale**2)
-
-    return mpi.mpi_sum_jax(O.T.conj() @ O)[0] + self.diag_shift * diag
+    return vec, lambda x: reassemble(unravel(x))
