@@ -22,14 +22,18 @@ from flax import struct
 from netket.utils.types import PyTree, Array
 from netket.utils import mpi
 import netket.jax as nkjax
+from netket.nn import split_array_mpi
 
 from ..linear_operator import LinearOperator, Uninitialized
 
 from .common import check_valid_vector_type
-from .qgt_jacobian_pytree_logic import mat_vec, prepare_centered_oks
-from .qgt_jacobian_common import choose_jacobian_mode
-
-from netket.nn import split_array_mpi
+from .qgt_jacobian_pytree_logic import mat_vec
+from .qgt_jacobian_common import (
+    choose_jacobian_mode,
+    sanitize_diag_shift,
+    to_shift_offset,
+    rescale,
+)
 
 
 def QGTJacobianPyTree(
@@ -37,7 +41,10 @@ def QGTJacobianPyTree(
     *,
     mode: str = None,
     holomorphic: bool = None,
-    rescale_shift=False,
+    diag_shift=None,
+    diag_scale=None,
+    rescale_shift=None,
+    chunk_size=None,
     **kwargs,
 ) -> "QGTJacobianPyTreeT":
     """
@@ -49,6 +56,18 @@ def QGTJacobianPyTree(
     The details on how the ⟨S⟩⁻¹⟨F⟩ system is solved are contained in
     the field `sr`.
 
+    Numerical estimates of the QGT are usually ill-conditioned and require
+    regularisation. The standard approach is to add a positive constant to the diagonal;
+    alternatively, Becca and Sorella (2017) propose scaling this offset with the
+    diagonal entry itself. NetKet allows using both in tandem:
+
+    .. math::
+
+        S_{ii} \\mapsto S_{ii} + \\epsilon_1 S_{ii} + \\epsilon_2;
+
+    :math:`\\epsilon_{1,2}` are specified using `diag_scale` and `diag_shift`,
+    respectively.
+
     Args:
         vstate: The variational state
         mode: "real", "complex" or "holomorphic": specifies the implementation
@@ -58,17 +77,31 @@ def QGTJacobianPyTree(
               models. holomorphic works for any function assuming it's holomorphic
               or real valued.
         holomorphic: a flag to indicate that the function is holomorphic.
-        rescale_shift: If True rescales the diagonal shift.
+        diag_scale: Fractional shift :math:`\\epsilon_1` added to diagonal entries (see above).
+        diag_shift: Constant shift :math:`\\epsilon_2` added to diagonal entries (see above).
+        chunk_size: If supplied, overrides the chunk size of the variational state
+                    (useful for models where the backward pass requires more
+                    memory than the forward pass).
     """
+    if mode is not None and holomorphic is not None:
+        raise ValueError("Cannot specify both `mode` and `holomorphic`.")
+    if rescale_shift is not None and diag_scale is not None:
+        raise ValueError("Cannot specify both `rescale_shift` and `diag_scale`.")
+
     if vstate is None:
         return partial(
             QGTJacobianPyTree,
             mode=mode,
             holomorphic=holomorphic,
+            chunk_size=chunk_size,
+            diag_shift=diag_shift,
+            diag_scale=diag_scale,
             rescale_shift=rescale_shift,
             **kwargs,
         )
 
+    diag_shift, diag_scale = sanitize_diag_shift(diag_shift, diag_scale, rescale_shift)
+
     # TODO: Find a better way to handle this case
     from netket.vqs import ExactState
 
@@ -86,30 +119,39 @@ def QGTJacobianPyTree(
             vstate.parameters,
             vstate.model_state,
             samples,
-            mode=mode,
             holomorphic=holomorphic,
         )
-    elif holomorphic is not None:
-        raise ValueError("Cannot specify both `mode` and `holomorphic`.")
 
-    if hasattr(vstate, "chunk_size"):
+    if chunk_size is None and hasattr(vstate, "chunk_size"):
         chunk_size = vstate.chunk_size
-    else:
-        chunk_size = None
 
-    O, scale = prepare_centered_oks(
+    shift, offset = to_shift_offset(diag_shift, diag_scale)
+
+    jacobians = nkjax.jacobian(
         vstate._apply_fun,
         vstate.parameters,
         samples.reshape(-1, samples.shape[-1]),
         vstate.model_state,
-        mode,
-        rescale_shift,
-        pdf,
-        chunk_size,
+        mode=mode,
+        pdf=pdf,
+        chunk_size=chunk_size,
+        dense=False,
+        center=True,
     )
 
+    if offset is not None:
+        ndims = 1 if mode != "complex" else 2
+        jacobians, scale = rescale(jacobians, offset, ndims=ndims)
+    else:
+        scale = None
+
     return QGTJacobianPyTreeT(
-        O=O, scale=scale, _params_structure=vstate.parameters, mode=mode, **kwargs
+        O=jacobians,
+        scale=scale,
+        _params_structure=vstate.parameters,
+        mode=mode,
+        diag_shift=shift,
+        **kwargs,
     )
 
 
@@ -264,7 +306,11 @@ def _solve(
 
 @jax.jit
 def _to_dense(self: QGTJacobianPyTreeT) -> jnp.ndarray:
-    O = jax.vmap(lambda l: nkjax.tree_ravel(l)[0])(self.O)
+    O = self.O
+    if self.mode == "complex":
+        # I want to iterate across the samples and real/imaginary part
+        O = jax.tree_map(lambda x: x.reshape(-1, *x.shape[2:]), O)
+    O = jax.vmap(lambda l: nkjax.tree_ravel(l)[0])(O)
 
     if self.scale is None:
         diag = jnp.eye(O.shape[1])
