@@ -21,7 +21,8 @@ from flax import struct
 
 import netket.jax as nkjax
 from netket.utils.types import PyTree
-from netket.utils import warn_deprecation
+
+from netket.nn import split_array_mpi
 
 from .common import check_valid_vector_type
 from .qgt_onthefly_logic import mat_vec_factory, mat_vec_chunked_factory
@@ -29,7 +30,7 @@ from .qgt_onthefly_logic import mat_vec_factory, mat_vec_chunked_factory
 from ..linear_operator import LinearOperator, Uninitialized
 
 
-def QGTOnTheFly(vstate=None, **kwargs) -> "QGTOnTheFlyT":
+def QGTOnTheFly(vstate=None, *, chunk_size=None, **kwargs) -> "QGTOnTheFlyT":
     """
     Lazy representation of an S Matrix computed by performing 2 jvp
     and 1 vjp products, using the variational state's model, the
@@ -42,28 +43,37 @@ def QGTOnTheFly(vstate=None, **kwargs) -> "QGTOnTheFlyT":
 
     Args:
         vstate: The variational State.
+        chunk_size: If supplied, overrides the chunk size of the variational state
+                    (useful for models where the backward pass requires more
+                    memory than the forward pass).
     """
     if vstate is None:
-        return partial(QGTOnTheFly, **kwargs)
+        return partial(QGTOnTheFly, chunk_size=chunk_size, **kwargs)
 
-    if "centered" in kwargs:
-        warn_deprecation(
-            "The argument `centered` is deprecated. The implementation now always behaves as if centered=False."
+    if kwargs.pop("diag_scale", None) is not None:
+        raise NotImplementedError(
+            "\n`diag_scale` argument is not yet supported by QGTOnTheFly."
+            "Please use `QGTJacobianPyTree` or `QGTJacobianDense`.\n\n"
+            "You are also encouraged to nag the developers to support "
+            "this feature.\n\n"
         )
-        kwargs.pop("centered")
 
     # TODO: Find a better way to handle this case
     from netket.vqs import ExactState
 
     if isinstance(vstate, ExactState):
-        raise TypeError("Only QGTJacobianPyTree works with ExactState.")
-
-    if jnp.ndim(vstate.samples) == 2:
-        samples = vstate.samples
+        samples = split_array_mpi(vstate._all_states)
+        pdf = split_array_mpi(vstate.probability_distribution())
     else:
-        samples = vstate.samples.reshape((-1, vstate.samples.shape[-1]))
+        if jnp.ndim(vstate.samples) == 2:
+            samples = vstate.samples
+        else:
+            samples = vstate.samples.reshape((-1, vstate.samples.shape[-1]))
+        pdf = None
+
+    if chunk_size is None and hasattr(vstate, "chunk_size"):
+        chunk_size = vstate.chunk_size
 
-    chunk_size = vstate.chunk_size
     n_samples = samples.shape[0]
 
     if chunk_size is None or chunk_size >= n_samples:
@@ -71,6 +81,8 @@ def QGTOnTheFly(vstate=None, **kwargs) -> "QGTOnTheFlyT":
         chunking = False
     else:
         samples, _ = nkjax.chunk(samples, chunk_size)
+        if pdf is not None:
+            pdf, _ = nkjax.chunk(pdf, chunk_size)
         mv_factory = mat_vec_chunked_factory
         chunking = True
 
@@ -79,6 +91,7 @@ def QGTOnTheFly(vstate=None, **kwargs) -> "QGTOnTheFlyT":
         params=vstate.parameters,
         model_state=vstate.model_state,
         samples=samples,
+        pdf=pdf,
     )
     return QGTOnTheFlyT(
         _mat_vec=mat_vec,
