@@ -25,7 +25,7 @@ from netket.hilbert import AbstractHilbert, ContinuousHilbert
 from netket.utils import mpi, wrap_afun
 from netket.utils.types import PyTree
 
-from netket.utils.deprecation import deprecated, warn_deprecation
+from netket.utils.deprecation import deprecated
 from netket.utils import struct
 
 from .base import Sampler, SamplerState
@@ -71,26 +71,6 @@ class MetropolisSamplerState(SamplerState):
 
         return self.n_accepted / self.n_steps
 
-    @property
-    @deprecated(
-        """Please use the attribute `.acceptance` instead of
-        `.acceptance_ratio`. The new attribute `.acceptance` returns the
-        acceptance ratio ∈ [0,1], instead of the current `acceptance_ratio`
-        returning a percentage, which is a bug."""
-    )
-    def acceptance_ratio(self):
-        """DEPRECATED: Please use the attribute `.acceptance` instead of
-        `.acceptance_ratio`. The new attribute `.acceptance` returns the
-        acceptance ratio ∈ [0,1], instead of the current `acceptance_ratio`
-        returning a percentage, which is a bug.
-
-        The percentage of accepted moves across all chains and MPI processes.
-
-        The rate is computed since the last reset of the sampler.
-        Will return None if no sampling has been performed since then.
-        """
-        return self.acceptance * 100
-
     @property
     def n_steps(self) -> int:
         """Total number of moves performed across all processes since the last reset."""
@@ -216,13 +196,6 @@ class MetropolisSampler(Sampler):
 
         kwargs["rule"] = rule
 
-        # deprecation warnings
-        if "reset_chain" in kwargs:
-            warn_deprecation(
-                "The keyword argument `reset_chain` is deprecated in favour of `reset_chains`"
-            )
-            kwargs["reset_chains"] = kwargs.pop("reset_chain")
-
         return args, kwargs
 
     def __post_init__(self):
@@ -451,7 +424,7 @@ def sample_next(
     return sampler.sample_next(machine, parameters, state)
 
 
-def MetropolisLocal(hilbert, *args, **kwargs) -> MetropolisSampler:
+def MetropolisLocal(hilbert, **kwargs) -> MetropolisSampler:
     r"""
     Sampler acting on one local degree of freedom.
 
@@ -488,11 +461,11 @@ def MetropolisLocal(hilbert, *args, **kwargs) -> MetropolisSampler:
     """
     from .rules import LocalRule
 
-    return MetropolisSampler(hilbert, LocalRule(), *args, **kwargs)
+    return MetropolisSampler(hilbert, LocalRule(), **kwargs)
 
 
 def MetropolisExchange(
-    hilbert, *args, clusters=None, graph=None, d_max=1, **kwargs
+    hilbert, *, clusters=None, graph=None, d_max=1, **kwargs
 ) -> MetropolisSampler:
     r"""
     This sampler acts locally only on two local degree of freedom :math:`s_i` and :math:`s_j`,
@@ -545,10 +518,10 @@ def MetropolisExchange(
     from .rules import ExchangeRule
 
     rule = ExchangeRule(clusters=clusters, graph=graph, d_max=d_max)
-    return MetropolisSampler(hilbert, rule, *args, **kwargs)
+    return MetropolisSampler(hilbert, rule, **kwargs)
 
 
-def MetropolisHamiltonian(hilbert, hamiltonian, *args, **kwargs) -> MetropolisSampler:
+def MetropolisHamiltonian(hilbert, hamiltonian, **kwargs) -> MetropolisSampler:
     r"""
     Sampling based on the off-diagonal elements of a Hamiltonian (or a generic Operator).
     In this case, the transition matrix is taken to be:
@@ -597,10 +570,10 @@ def MetropolisHamiltonian(hilbert, hamiltonian, *args, **kwargs) -> MetropolisSa
     from .rules import HamiltonianRule
 
     rule = HamiltonianRule(hamiltonian)
-    return MetropolisSampler(hilbert, rule, *args, **kwargs)
+    return MetropolisSampler(hilbert, rule, **kwargs)
 
 
-def MetropolisGaussian(hilbert, sigma=1.0, *args, **kwargs) -> MetropolisSampler:
+def MetropolisGaussian(hilbert, sigma=1.0, **kwargs) -> MetropolisSampler:
     """This sampler acts on all particle positions simultaneously
     and proposes a new state according to a Gaussian distribution
     with width `sigma`.
@@ -622,4 +595,39 @@ def MetropolisGaussian(hilbert, sigma=1.0, *args, **kwargs) -> MetropolisSampler
     from .rules import GaussianRule
 
     rule = GaussianRule(sigma)
-    return MetropolisSampler(hilbert, rule, *args, **kwargs)
+    return MetropolisSampler(hilbert, rule, **kwargs)
+
+
+def MetropolisAdjustedLangevin(
+    hilbert, dt=0.001, chunk_size=None, **kwargs
+) -> MetropolisSampler:
+    r"""This sampler acts on all particle positions simultaneously
+    and takes a Langevin step [1]:
+
+    .. math::
+       x_{t+dt} = x_t + dt \nabla_x \log p(x) \vert_{x=x_t} + \sqrt{2 dt}\eta,
+
+    where  :math:`\eta` is normal distributed noise :math:`\eta \sim \mathcal{N}(0,1)`.
+    This sampler only works for continuous Hilbert spaces.
+
+    [1]: https://en.wikipedia.org/wiki/Metropolis-adjusted_Langevin_algorithm
+
+    Args:
+        hilbert: The continuous Hilbert space to sample.
+        dt: Time step size for the Langevin dynamics (noise with variance 2*dt).
+        chunk_size: Chunk size to compute the gradients of the log probability.
+        n_chains: The total number of independent Markov chains across all MPI ranks. Either specify this or `n_chains_per_rank`.
+        n_chains_per_rank: Number of independent chains on every MPI rank (default = 16).
+        n_sweeps: Number of sweeps for each step along the chain. Defaults to the number of sites in the Hilbert space.
+                This is equivalent to subsampling the Markov chain.
+        reset_chains: If True, resets the chain state when `reset` is called on every new sampling (default = False).
+        machine_pow: The power to which the machine should be exponentiated to generate the pdf (default = 2).
+        dtype: The dtype of the states sampled (default = np.float64).
+    """
+    if not isinstance(hilbert, ContinuousHilbert):
+        raise ValueError("This sampler only works for Continuous Hilbert spaces.")
+
+    from .rules import LangevinRule
+
+    rule = LangevinRule(dt=dt, chunk_size=chunk_size)
+    return MetropolisSampler(hilbert, rule, **kwargs)
