@@ -19,6 +19,7 @@ import inspect
 from textwrap import dedent
 
 from .config_flags import config
+from .version_check import module_version
 
 
 def deprecated(reason=None, func_name=None):
@@ -101,9 +102,9 @@ def deprecate_dtype(clz):
     This decorator also adds a deprecated field `dtype` to the class, which
     returns `param_dtype`.
     """
-    # Sphinx is terrible at understanding wrappers of classses, so we detect
+    # Sphinx is terrible at understanding wrappers of classes, so we detect
     # this env variable set by us sphinx's conf.py, and we do not apply the
-    # deecorator if that's the case.
+    # decorator if that's the case.
     if config.netket_sphinx_build:
         return clz
 
@@ -133,3 +134,17 @@ def deprecate_dtype(clz):
         return res
 
     return helper
+
+
+# TODO: remove the switch when we support only jax >= 0.3.17
+
+
+def pure_callback(callback, result_shape_dtypes, *args):
+    if module_version("jax") >= (0, 3, 17):
+        from jax import pure_callback
+
+        return pure_callback(callback, result_shape_dtypes, *args)
+    else:
+        from jax.experimental.host_callback import call
+
+        return call(callback, *args, result_shape=result_shape_dtypes)
