@@ -14,6 +14,7 @@
 
 from typing import Any, Sequence, Callable, Union
 
+import optax as _optax
 import jax as _jax
 import jaxlib as _jaxlib
 import numpy as _np
@@ -24,7 +25,13 @@ SeedT = Union[int, PRNGKeyT]
 Shape = Sequence[int]
 DType = Any  # this could be a real type?
 
-Array = Union[_np.ndarray, _jaxlib.xla_extension.DeviceArray, _jax.core.Tracer]
+if not hasattr(_jax, "Array"):
+    # pre jax 0.4
+    Array = Union[_np.ndarray, _jaxlib.xla_extension.DeviceArray, _jax.core.Tracer]
+else:
+    # TODO keep only this after jax>=0.4 is required
+    Array = Union[_np.ndarray, _jax.Array]
+
 ArrayLike = Any  # Objects that are valid inputs to (np|jnp).asarray.
 
 NNInitFunc = Callable[[PRNGKeyT, Shape, DType], Array]
@@ -32,3 +39,5 @@ NNInitFunc = Callable[[PRNGKeyT, Shape, DType], Array]
 PyTree = Any
 
 Scalar = Any
+
+ScalarOrSchedule = Union[Scalar, _optax.Schedule]
