@@ -13,10 +13,12 @@
 # limitations under the License.
 
 from functools import partial, lru_cache
-from typing import Callable, Any, Tuple
+from typing import Callable, Tuple
 
 import jax
 from jax import numpy as jnp
+from flax.core.scope import CollectionFilter
+
 from netket import jax as nkjax
 from netket.operator import Squared
 from netket.stats import Stats
@@ -68,7 +70,7 @@ def expect_and_forces(
     vstate: ExactState,
     Ô: DiscreteOperator,
     *,
-    mutable: Any,
+    mutable: CollectionFilter,
 ) -> Tuple[Stats, PyTree]:
     if isinstance(Ô, Squared):
         raise NotImplementedError("expect_and_forces not yet implemented for `Squared`")
@@ -79,7 +81,7 @@ def expect_and_forces(
     Ψ = vstate.to_array()
     OΨ = O @ Ψ
 
-    _, Ō_grad, expval_O, new_model_state = _exp_forces(
+    expval_O, Ō_grad, new_model_state = _exp_forces(
         vstate._apply_fun,
         mutable,
         vstate.parameters,
@@ -98,7 +100,7 @@ def expect_and_forces(
 @partial(jax.jit, static_argnums=(0, 1))
 def _exp_forces(
     model_apply_fun: Callable,
-    mutable: bool,
+    mutable: CollectionFilter,
     parameters: PyTree,
     model_state: PyTree,
     σ: jnp.ndarray,
@@ -108,6 +110,8 @@ def _exp_forces(
     is_mutable = mutable is not False
 
     expval_O = (Ψ.conj() * OΨ).sum()
+    variance = jnp.sum(jnp.abs(OΨ - expval_O * Ψ) ** 2)
+
     ΔOΨ = (OΨ - expval_O * Ψ).conj() * Ψ
 
     _, vjp_fun, *new_model_state = nkjax.vjp(
@@ -122,9 +126,8 @@ def _exp_forces(
     new_model_state = new_model_state[0] if is_mutable else None
 
     return (
-        None,
+        Stats(mean=expval_O, error_of_mean=0.0, variance=variance),
         Ō_grad,
-        expval_O,
         new_model_state,
     )
 
@@ -135,7 +138,7 @@ def expect_and_grad(
     Ô: DiscreteOperator,
     use_covariance: TrueT,
     *,
-    mutable: Any,
+    mutable: CollectionFilter,
 ) -> Tuple[Stats, PyTree]:
     Ō, Ō_grad = expect_and_forces(vstate, Ô, mutable=mutable)
     Ō_grad = _force_to_grad(Ō_grad, vstate.parameters)
