@@ -23,7 +23,6 @@ import netket
 from netket import jax as nkjax
 from netket.sampler import Sampler
 from netket.stats import Stats
-from netket.utils import warn_deprecation
 from netket.utils.types import PyTree
 from netket.operator import AbstractOperator
 
@@ -53,7 +52,6 @@ class MCMixedState(VariationalMixedState, MCState):
         n_samples_diag: int = None,
         n_samples_per_rank_diag: Optional[int] = None,
         n_discard_per_chain_diag: Optional[int] = None,
-        n_discard_diag: Optional[int] = None,  # deprecated
         seed=None,
         sampler_seed: Optional[int] = None,
         variables=None,
@@ -79,7 +77,7 @@ class MCMixedState(VariationalMixedState, MCState):
             parameters: Optional PyTree of weights from which to start.
             seed: rng seed used to generate a set of parameters (only if parameters is not passed). Defaults to a random one.
             sampler_seed: rng seed used to initialise the sampler. Defaults to a random one.
-            mutable: Dict specifying mutable arguments. Use it to specify if the model has a state that can change
+            mutable: Name or list of names of mutable arguments. Use it to specify if the model has a state that can change
                 during evaluation, but that should not be optimised. See also flax.linen.module.apply documentation
                 (default=False)
             init_fun: Function of the signature f(model, shape, rng_key, dtype) -> Optional_state, parameters used to
@@ -123,25 +121,11 @@ class MCMixedState(VariationalMixedState, MCState):
 
         for kw in [
             "n_samples",
-            "n_discard",
             "n_discard_per_chain",
-        ]:  # TODO remove n_discard after deprecation.
+        ]:
             if kw in kwargs:
                 kwargs.pop(kw)
 
-        # TODO: remove deprecation.
-        if n_discard_diag is not None and n_discard_per_chain_diag is not None:
-            raise ValueError(
-                "`n_discard_diag` has been renamed to `n_discard_per_chain_diag` and deprecated."
-                "Specify only `n_discard_per_chain_diag`."
-            )
-        elif n_discard_diag is not None:
-            warn_deprecation(
-                "`n_discard_diag` has been renamed to `n_discard_per_chain_diag` and deprecated."
-                "Please update your code to `n_discard_per_chain_diag`."
-            )
-            n_discard_per_chain_diag = n_discard_diag
-
         self._diagonal = MCState(
             sampler_diag,
             apply_fun=diagonal_apply_fun,
@@ -154,11 +138,6 @@ class MCMixedState(VariationalMixedState, MCState):
             **kwargs,
         )
 
-        # build the
-
-    # def init(self, *args, **kwargs):
-    #    super().init(*args, **kwargs)
-
     @property
     def diagonal(self):
         return self._diagonal
@@ -209,29 +188,6 @@ class MCMixedState(VariationalMixedState, MCState):
     def n_discard_per_chain_diag(self, n_discard_per_chain: Optional[int]):
         self.diagonal.n_discard_per_chain = n_discard_per_chain
 
-    # TODO: deprecate
-    @property
-    def n_discard_diag(self) -> int:
-        """
-        DEPRECATED: Use `n_discard_per_chain_diag` instead.
-
-        Number of discarded samples at the beginning of the markov chain.
-        """
-        warn_deprecation(
-            "`n_discard_diag` has been renamed to `n_discard_per_chain_diag` and deprecated."
-            "Please update your code to use `n_discard_per_chain_diag`."
-        )
-
-        return self.n_discard_per_chain_diag
-
-    @n_discard_diag.setter
-    def n_discard_diag(self, val) -> int:
-        warn_deprecation(
-            "`n_discard_diag` has been renamed to `n_discard_per_chain_diag` and deprecated."
-            "Please update your code to use `n_discard_per_chain_diag`."
-        )
-        self.n_discard_per_chain_diag = val
-
     @MCState.parameters.setter
     def parameters(self, pars: PyTree):
         MCState.parameters.fset(self, pars)
