@@ -23,6 +23,7 @@ from jax import numpy as jnp
 
 import flax
 from flax import serialization
+from flax.core.scope import CollectionFilter
 
 from netket import jax as nkjax
 from netket import nn
@@ -31,8 +32,6 @@ from netket.operator import AbstractOperator, Squared
 from netket.sampler import Sampler, SamplerState
 from netket.utils import (
     maybe_wrap_module,
-    deprecated,
-    warn_deprecation,
     mpi,
     wrap_afun,
     wrap_to_support_scalar,
@@ -138,7 +137,6 @@ class MCState(VariationalState):
         *,
         n_samples: int = None,
         n_samples_per_rank: Optional[int] = None,
-        n_discard: Optional[int] = None,  # deprecated
         n_discard_per_chain: Optional[int] = None,
         chunk_size: Optional[int] = None,
         variables: Optional[PyTree] = None,
@@ -147,7 +145,7 @@ class MCState(VariationalState):
         sample_fun: Callable = None,
         seed: Optional[SeedT] = None,
         sampler_seed: Optional[SeedT] = None,
-        mutable: bool = False,
+        mutable: CollectionFilter = False,
         training_kwargs: Dict = {},
     ):
         """
@@ -164,7 +162,7 @@ class MCState(VariationalState):
             parameters: Optional PyTree of weights from which to start.
             seed: rng seed used to generate a set of parameters (only if parameters is not passed). Defaults to a random one.
             sampler_seed: rng seed used to initialise the sampler. Defaults to a random one.
-            mutable: Dict specifying mutable arguments. Use it to specify if the model has a state that can change
+            mutable: Name or list of names of mutable arguments. Use it to specify if the model has a state that can change
                 during evaluation, but that should not be optimised. See also flax.linen.module.apply documentation
                 (default=False)
             init_fun: Function of the signature f(model, shape, rng_key, dtype) -> Optional_state, parameters used to
@@ -176,7 +174,6 @@ class MCState(VariationalState):
             sample_fun: Optional function used to sample the state, if it is not the same as `apply_fun`.
             training_kwargs: a dict containing the optional keyword arguments to be passed to the apply_fun during training.
                 Useful for example when you have a batchnorm layer that constructs the average/mean only during training.
-            n_discard: DEPRECATED. Please use `n_discard_per_chain` which has the same behaviour.
         """
         super().__init__(sampler.hilbert)
 
@@ -229,18 +226,6 @@ class MCState(VariationalState):
                 "can be specified at the same time."
             )
 
-        if n_discard is not None and n_discard_per_chain is not None:
-            raise ValueError(
-                "`n_discard` has been renamed to `n_discard_per_chain` and deprecated."
-                "Specify only `n_discard_per_chain`."
-            )
-        elif n_discard is not None:
-            warn_deprecation(
-                "`n_discard` has been renamed to `n_discard_per_chain` and deprecated."
-                "Please update your code to use `n_discard_per_chain`."
-            )
-            n_discard_per_chain = n_discard
-
         if sample_fun is not None:
             self._sample_fun = sample_fun
         else:
@@ -405,29 +390,6 @@ class MCState(VariationalState):
             else self.n_samples // 10
         )
 
-    # TODO: deprecate
-    @property
-    def n_discard(self) -> int:
-        """
-        DEPRECATED: Use `n_discard_per_chain` instead.
-
-        Number of discarded samples at the beginning of the markov chain.
-        """
-        warn_deprecation(
-            "`n_discard` has been renamed to `n_discard_per_chain` and deprecated."
-            "Please update your code to use `n_discard_per_chain`."
-        )
-
-        return self.n_discard_per_chain
-
-    @n_discard.setter
-    def n_discard(self, val) -> int:
-        warn_deprecation(
-            "`n_discard` has been renamed to `n_discard_per_chain` and deprecated."
-            "Please update your code to use `n_discard_per_chain`."
-        )
-        self.n_discard_per_chain = val
-
     @property
     def chunk_size(self) -> int:
         """
@@ -606,7 +568,7 @@ class MCState(VariationalState):
         self,
         Ô: AbstractOperator,
         *,
-        mutable: Optional[Any] = None,
+        mutable: Optional[CollectionFilter] = None,
         use_covariance: Optional[bool] = None,
     ) -> Tuple[Stats, PyTree]:
         r"""Estimates the quantum expectation value and its gradient for a given operator O.
@@ -639,7 +601,7 @@ class MCState(VariationalState):
         self,
         Ô: AbstractOperator,
         *,
-        mutable: Optional[Any] = None,
+        mutable: Optional[CollectionFilter] = None,
     ) -> Tuple[Stats, PyTree]:
         r"""Estimates the quantum expectation value and the corresponding force vector for a given operator O.
 
@@ -672,13 +634,6 @@ class MCState(VariationalState):
 
         return expect_and_forces(self, Ô, self.chunk_size, mutable=mutable)
 
-    @deprecated("Use MCState.log_value(σ) instead.")
-    def evaluate(self, σ: jnp.ndarray) -> jnp.ndarray:
-        """
-        DEPRECATED: use log_value instead.
-        """
-        return self.log_value(σ)
-
     def quantum_geometric_tensor(
         self, qgt_T: LinearOperator = QGTAuto()
     ) -> LinearOperator:
